import pytest
from steamship_tests.utils.client import TESTING_PROFILE
from steamship_tests.utils.fixtures import get_steamship_client
from steamship_tests.utils.random import random_name

from steamship import Steamship, SteamshipError, Workspace
from steamship.data.user import User


def test_connect():
    """Test basic connection"""
    client = get_steamship_client()
    assert client.config is not None
    assert client.config.profile == TESTING_PROFILE
    assert client.config.api_key is not None


def test_user():
    client = get_steamship_client()
    user = User.current(client)
    assert user.id is not None
    assert user.handle is not None


def test_temporary_workspace():
    with Steamship.temporary_workspace(profile=TESTING_PROFILE) as client:
        workspace_handle = client.config.workspace_handle
        assert workspace_handle.index("temp-") == 0
        workspace = Workspace.get(client, handle=workspace_handle)
        assert workspace.handle == workspace_handle

    with Steamship.temporary_workspace(profile=TESTING_PROFILE) as client:
        # Now it's deleted. It can't be fetched.
        with pytest.raises(SteamshipError):
            Workspace.get(client, handle=workspace_handle)


def test_temporary_workspace_with_workspace_switch():
    with Steamship.temporary_workspace(profile=TESTING_PROFILE) as client:
        workspace_handle = client.config.workspace_handle
        assert workspace_handle.index("temp-") == 0
        workspace = Workspace.get(client, handle=workspace_handle)
        assert workspace.handle == workspace_handle

        # Now we switch the workspace!
        client.switch_workspace(workspace_handle=random_name())

        # The new workspace handle is different
        new_workspace_handle = client.config.workspace_handle
        assert new_workspace_handle != workspace_handle

    # The original workspace is still deleted
    with Steamship.temporary_workspace(profile=TESTING_PROFILE) as client:
        # Now it's deleted. It can't be fetched.
        with pytest.raises(SteamshipError):
            Workspace.get(client, handle=workspace_handle)

        # But we CAN fetch the one we switched to. It was not auto-deleted.
        new_workspace = Workspace.get(client, handle=new_workspace_handle)
        # But delete it for good measure.
        new_workspace.delete()


def test_client_has_default_workspace_unless_otherwise_specified():
    client1 = get_steamship_client()
    assert client1.config.workspace_handle == "default"


def test_client_can_create_new_workspace():
    # Create a new workspace anchored in a randomly generated workspace name
    workspace_handle = random_name()
    custom_client = get_steamship_client(workspace_handle=workspace_handle)

    # The custom_client is not in the default workspace
    default_client = get_steamship_client()
    assert custom_client.config.workspace_handle == workspace_handle
    assert custom_client.config.workspace_id != default_client.config.workspace_id

    # Another client specifying the same workspace will be anchored to that workspace.
    custom_client_2 = get_steamship_client(workspace_handle=workspace_handle)
    assert custom_client.config.workspace_handle == custom_client_2.config.workspace_handle
    assert custom_client.config.workspace_id == custom_client_2.config.workspace_id

    # .. But if we specify that workspace with the `fail_if_workspace_exists` option it will fail.
    with pytest.raises(SteamshipError):
        get_steamship_client(
            workspace_handle=custom_client.config.workspace_handle, fail_if_workspace_exists=True
        )

    Workspace(client=custom_client, id=custom_client.config.workspace_id).delete()


def test_switch_workspace():
    """Tests that the client actively loads the default workspace ID and Handle, and that we can revert to it later."""
    default_client = get_steamship_client()
    custom_client = get_steamship_client(workspace_handle=random_name())
    custom_client_2 = get_steamship_client(workspace_handle=custom_client.config.workspace_handle)

    assert custom_client.config.workspace_handle != default_client.config.workspace_handle
    assert custom_client_2.config.workspace_handle == custom_client.config.workspace_handle
    assert default_client.config.workspace_handle == "default"

    # Switch custom_client_2 to the default
    custom_client_2.switch_workspace()
    assert custom_client_2.config.workspace_handle == default_client.config.workspace_handle
    assert custom_client_2.config.workspace_id == default_client.config.workspace_id

    # Switch default_workspace to the custom_client
    default_client.switch_workspace(workspace_handle=custom_client.config.workspace_handle)
    assert custom_client.config.workspace_handle == default_client.config.workspace_handle
    assert custom_client.config.workspace_id == default_client.config.workspace_id

    # client2 has the newly created workspace.
    Workspace(client=custom_client, id=custom_client.config.workspace_id).delete()



from __future__ import annotations

from steamship.base.request import Request
from steamship.base.response import Response

from ..file import File


class TagRequest(Request):
    type: str = None
    id: str = None
    name: str = None
    handle: str = None
    plugin_instance: str = None
    file: File = None


class TagResponse(Response):
    file: File = None



import logging
from abc import ABC, abstractmethod

from steamship.invocable import InvocableResponse, post
from steamship.invocable.plugin_service import PluginRequest, PluginService, TrainablePluginService
from steamship.plugin.inputs.block_and_tag_plugin_input import BlockAndTagPluginInput
from steamship.plugin.inputs.train_plugin_input import TrainPluginInput
from steamship.plugin.inputs.training_parameter_plugin_input import TrainingParameterPluginInput
from steamship.plugin.outputs.block_and_tag_plugin_output import BlockAndTagPluginOutput
from steamship.plugin.outputs.train_plugin_output import TrainPluginOutput
from steamship.plugin.outputs.training_parameter_plugin_output import TrainingParameterPluginOutput
from steamship.plugin.trainable_model import TrainableModel

# Note!
# =====
#
# This is the PLUGIN IMPLEMENTOR's View of a Tagger.
#
# If you are using the Steamship Client, you probably want steamship.client.operations.tagger instead
# of this file.
#


class Tagger(PluginService[BlockAndTagPluginInput, BlockAndTagPluginOutput], ABC):
    @abstractmethod
    def run(
        self, request: PluginRequest[BlockAndTagPluginInput]
    ) -> InvocableResponse[BlockAndTagPluginOutput]:
        raise NotImplementedError()

    @post("tag")
    def run_endpoint(self, **kwargs) -> InvocableResponse[BlockAndTagPluginOutput]:
        """Exposes the Tagger's `run` operation to the Steamship Engine via the expected HTTP path POST /tag"""
        return self.run(PluginRequest[BlockAndTagPluginInput].parse_obj(kwargs))


class TrainableTagger(TrainablePluginService[BlockAndTagPluginInput, BlockAndTagPluginOutput], ABC):
    @abstractmethod
    def run_with_model(
        self, request: PluginRequest[BlockAndTagPluginInput], model: TrainableModel
    ) -> InvocableResponse[BlockAndTagPluginOutput]:
        raise NotImplementedError()

    # noinspection PyUnusedLocal
    @post("tag")
    def run_endpoint(self, **kwargs) -> InvocableResponse[BlockAndTagPluginOutput]:
        """Exposes the Tagger's `run` operation to the Steamship Engine via the expected HTTP path POST /tag"""
        return self.run(PluginRequest[BlockAndTagPluginInput].parse_obj(kwargs))

    # noinspection PyUnusedLocal
    @post("getTrainingParameters")
    def get_training_parameters_endpoint(
        self, **kwargs
    ) -> InvocableResponse[TrainingParameterPluginOutput]:
        """Exposes the Service's `get_training_parameters` operation to the Steamship Engine via the expected HTTP path POST /getTrainingParameters"""
        return self.get_training_parameters(PluginRequest[TrainingParameterPluginInput](**kwargs))

    # noinspection PyUnusedLocal
    @post("train")
    def train_endpoint(self, **kwargs) -> InvocableResponse[TrainPluginOutput]:
        """Exposes the Service's `train` operation to the Steamship Engine via the expected HTTP path POST /train"""
        logging.info(f"Tagger:train_endpoint called. Calling train {kwargs}")
        arg = PluginRequest[TrainPluginInput].parse_obj(kwargs)
        model = self.model_cls()()
        model.receive_config(config=self.config)

        if arg.is_status_check:
            return self.train_status(arg, model)
        else:
            return self.train(arg, model)
